# Copyright 2007 Arnaud Cornet.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License, version 2.1 as published by the Free Software Foundation.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA  02110-1301  USA.

require 'vlcglue'
module VLC
  private
  class Input
    def length
      input_get_length
    end

    def time
      input_get_time
    end

    def time=(t)
      input_set_time(t)
    end

    def position
      input_get_position
    end

    def position=(pos)
      input_set_position(pos)
    end

    def will_play?
      input_will_play
    end

    def rate
      input_get_rate
    end

    def rate=(r)
      input_set_rate(r)
    end

    def state
      input_get_state
    end

    def has_vout?
      input_has_vout
    end

    def fps
      input_get_fps
    end

    def toggle_fullscreen
      toggle_fullscreen
    end

    def fullscreen=(b)
      set_fullscreen(b)
    end

    def fullscreen
      get_fullscreen
    end

    def video_width
      video_get_width
    end

    def video_height
      video_get_height
    end

    def take_snapshot(path)
      video_take_snapshot(path)
    end

    def video_resize(width, height)
      video_resize(width, height)
    end

    def video_reparent(drawable)
      video_reparent(drawable)
    end
  end

  class Playlist
    def initialize(instance)
      @instance = instance
    end

    def play(idx = -1, options = nil)
      @instance.playlist_play(idx, options)
    end

    def pause
      @instance.playlist_pause
    end

    def playing?
      @instance.playlist_isplaying
    end

    def size
      @instance.playlist_items_count
    end
    
    def stop
      @instance.playlist_stop
    end

    def next
      @instance.playlist_next
    end

    def prev
      @instance.playlist_prev
    end

    def clear
      @instance.playlist_clear
    end


    def add(mrl, name, options = nil)
      if options
        @instance.playlist_add_extended(mrl, name, options)
      else
        @instance.playlist_add(mrl, name)
      end
    end

    def delete(idx)
      @instance.playlist_delete_item(idx)
    end

    def input
      @instance.playlist_get_input
    end
  end

  class VLM
    def initialize(instance, name)
      @instance, @name = instance, name
    end

    def enabled=(b)
      @instance.vlm_set_enabled(@name, b)
    end

    def output=(outmrl)
      @instance.vlm_set_output(@name, outmrl)
    end

    def input=(inmrl)
      @instance.vlm_set_input(@name, inmrl)
    end

    def loop=(bool)
      @instance.vlm_set_loop(@name, bool)
    end

    def change_media(inputmrl, outputmrl, options, enabled, lOOp)
      @instance.vlm_change_media(@name, inputmrl, outputmrl, options, enabled,
                                lOOp)
    end

    def play
      @instance.vlm_play_media(@name)
    end

    def stop
      @instance.vlm_stop_media(@name)
    end

    def pause
      @instance.vlm_pause_media(@name)
    end
  end

  class Instance
    attr_reader :playlist
    def initialize
      @playlist = Playlist.new(self)
      @vlm = Hash.new
    end

    alias video_parent= video_set_parent
    alias video_viewport= video_set_viewport
    alias audio_mute= audio_set_mute
    alias audio_volume audio_get_volume
    alias audio_volume= audio_set_volume

    def vlm_broadcast_new(name, inputmrl, outputmrl, options, enabled = true,
                          lOOp = false)
      if @vlm.member?(name)
        raise RuntimeExeption, "VLM '#{name}' already exists"
      end

      vlm = VLM.new(self, name)
      @vlm[name] = vlm

      vlm_add_broadcast(name, inputmrl, outputmrl, options, enabled, lOOp)
      vlm
    end

    def vlm_delete(name)
      vlm_del_media(name)
      @vlm.delete(name)
    end
  end
end

